@@ -1,21 +1,30 @@
+from concurrent.futures.process import _threads_wakeups
+from pickle import FALSE
+import logging, logging.handlers, sys
 import click
-import click_logging
-import logging
+import click_log
 import shutil
 import datetime
 import os, zipfile, requests, io
 import tempfile, shutil
 import platform
 from pathlib import Path
-import sys
 from typing import Optional
 
-from . import utils, static, VERSION, CORE_COMMIT, core
-from .project import Target, Project
+from . import utils, templates, VERSION, CORE_COMMIT
+from .project import Project
 
 
 log = logging.getLogger('ptxlogger')
-click_logging.basic_config(log)
+click_log.basic_config(log)
+click_log_format = click_log.ColorFormatter()
+# create memory handler which displays error and critical messages at the end as well.
+sh = logging.StreamHandler(sys.stdout)
+sh.setFormatter(click_log_format)
+mh = logging.handlers.MemoryHandler(capacity=1024*100, flushLevel=100,target=sh, flushOnClose=True)
+mh.setLevel(logging.ERROR)
+log.addHandler(mh)
+
 
 CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
 
@@ -24,7 +33,7 @@ CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
 @click.group(invoke_without_command=True, context_settings=CONTEXT_SETTINGS)
 @click.pass_context
 # Allow a verbosity command:
-@click_logging.simple_verbosity_option(
+@click_log.simple_verbosity_option(
     log,
     help="Sets the severity of log messaging: DEBUG for all, INFO (default) for most, then WARNING, ERROR, and CRITICAL for decreasing verbosity."
 )
@@ -45,14 +54,18 @@ def main(ctx,targets):
     Use the `--help` option on any CLI command to learn more, for example,
     `pretext build --help`.
     """
-    if targets:
-        Project().print_target_names()
-        return
+    log.warning("PreTeXt-CLI is moving! To upgrade to version 1.0.0 or higher:")
+    log.warning("    pip uninstall pretextbook")
+    log.warning("    pip install --upgrade pretext")
+    log.warning("")
     if utils.project_path() is not None:
+        if targets:
+            Project().print_target_names()
+            return
         # create file handler which logs even debug messages
         fh = logging.FileHandler(utils.project_path()/'cli.log', mode='w')
         fh.setLevel(logging.DEBUG)
-        file_log_format = logging.Formatter('%(levelname)-8s:  %(message)s')
+        file_log_format = logging.Formatter('{levelname:<8}: {message}',style='{')
         fh.setFormatter(file_log_format)
         log.addHandler(fh)
         # output info
@@ -110,6 +123,7 @@ def support():
                     f"Unable to locate the command for <{exec_name}> on your system.")
     else:
         log.info("No project.ptx found.")
+    log.info("")
 
 
 # pretext new
@@ -137,8 +151,8 @@ def new(template,directory,url_template):
         r = requests.get(url_template)
         archive = zipfile.ZipFile(io.BytesIO(r.content))
     else:
-        template_path = static.path('templates', f'{template}.zip')
-        archive = zipfile.ZipFile(template_path)
+        with templates.resource_path(f'{template}.zip') as template_path:
+            archive = zipfile.ZipFile(template_path)
     # find (first) project.ptx to use as root of template
     filenames = [Path(filepath).name for filepath in archive.namelist()]
     project_ptx_index = filenames.index('project.ptx')
@@ -154,6 +168,7 @@ def new(template,directory,url_template):
         f.write(f"pretextbook == {VERSION}")
     log.info(f"Success! Open `{directory_fullpath}/source/main.ptx` to edit your document")
     log.info(f"Then try to `pretext build` and `pretext view` from within `{directory_fullpath}`.")
+    log.info("")
 
 
 # pretext init
@@ -175,15 +190,15 @@ def init(refresh):
         log.warning(f"Use `pretext init --refresh` to refresh initialization of an existing project.")
         return
     timestamp = datetime.datetime.now().strftime('%Y%m%d-%H%M%S')
-    template_manifest_path = static.path('templates', 'project.ptx')
-    project_manifest_path = Path("project.ptx").resolve()
-    if project_manifest_path.is_file():
-        project_manifest_path = Path(f'project-{timestamp}.ptx').resolve()
-        log.warning(f"You already have a project file at {Path('project.ptx').resolve()}.")
-        log.warning(f"A default project file has been created as {project_manifest_path} for comparison.")
-    log.info(f"Generated project file at `{project_manifest_path}`.")
-    log.info("")
-    shutil.copyfile(template_manifest_path,project_manifest_path)
+    with templates.resource_path('project.ptx') as template_manifest_path:
+        project_manifest_path = Path("project.ptx").resolve()
+        if project_manifest_path.is_file():
+            project_manifest_path = Path(f'project-{timestamp}.ptx').resolve()
+            log.warning(f"You already have a project file at {Path('project.ptx').resolve()}.")
+            log.warning(f"A default project file has been created as {project_manifest_path} for comparison.")
+        log.info(f"Generated project file at `{project_manifest_path}`.")
+        log.info("")
+        shutil.copyfile(template_manifest_path,project_manifest_path)
     # Create requirements.txt
     requirements_path = Path('requirements.txt').resolve()
     if requirements_path.exists():
@@ -195,31 +210,32 @@ def init(refresh):
     log.info(f"Generated requirements file at {requirements_path}.")
     log.info("")
     # Create publication file if one doesn't exist: 
-    template_pub_path = static.path('templates','publication.ptx')
-    pub_dir_path = Path('publication')
-    if not pub_dir_path.exists():
-        pub_dir_path.mkdir()
-    project_pub_path = (pub_dir_path/'publication.ptx').resolve()
-    if project_pub_path.exists():
-        project_pub_path = (Path('publication')/f'publication-{timestamp}.ptx').resolve()
-        log.warning(f"You already have a publication file at {(Path('publication')/'publication.ptx').resolve()}.")
-        log.warning(f"A default project file has been created as {project_pub_path} for comparison.")
-    shutil.copyfile(template_pub_path, project_pub_path)
-    log.info(f"Generated publication file at {project_pub_path}.")
-    log.info("")
+    with templates.resource_path("publication.ptx") as template_pub_path:
+        pub_dir_path = Path('publication')
+        if not pub_dir_path.exists():
+            pub_dir_path.mkdir()
+        project_pub_path = (pub_dir_path/'publication.ptx').resolve()
+        if project_pub_path.exists():
+            project_pub_path = (Path('publication')/f'publication-{timestamp}.ptx').resolve()
+            log.warning(f"You already have a publication file at {(Path('publication')/'publication.ptx').resolve()}.")
+            log.warning(f"A default project file has been created as {project_pub_path} for comparison.")
+        shutil.copyfile(template_pub_path, project_pub_path)
+        log.info(f"Generated publication file at {project_pub_path}.")
+        log.info("")
     # Create .gitignore if one doesn't exist
-    template_gitignore_path = static.path('templates','.gitignore')
-    project_gitignore_path = Path(".gitignore").resolve()
-    if project_gitignore_path.exists():
-        project_gitignore_path = Path(f".gitignore-{timestamp}").resolve()
-        log.warning(f"You already have a gitignore file at {Path('.gitignore').resolve()}.")
-        log.warning(f"A default project file has been created as {project_gitignore_path} for comparison.")
-    shutil.copyfile(template_gitignore_path,project_gitignore_path)
+    with templates.resource_path('.gitignore') as template_gitignore_path:
+        project_gitignore_path = Path(".gitignore").resolve()
+        if project_gitignore_path.exists():
+            project_gitignore_path = Path(f".gitignore-{timestamp}").resolve()
+            log.warning(f"You already have a gitignore file at {Path('.gitignore').resolve()}.")
+            log.warning(f"A default project file has been created as {project_gitignore_path} for comparison.")
+        shutil.copyfile(template_gitignore_path,project_gitignore_path)
     log.info(f"Generated .gitignore file at {project_gitignore_path}.")
     log.info("")
     # End by reporting success
     log.info(f"Success! Open project.ptx to edit your project manifest.")
     log.info(f"Edit your <target/>s to point to the location of your PreTeXt source files.")
+    log.info("")
 
 ASSETS = ['ALL', 'webwork', 'latex-image', 'sageplot', 'asymptote', 'interactive', 'youtube', 'codelens']
 
@@ -231,23 +247,8 @@ ASSETS = ['ALL', 'webwork', 'latex-image', 'sageplot', 'asymptote', 'interactive
 @click.option(
     '-g', '--generate', is_flag=False, flag_value="ALL", default=None,
     type=click.Choice(ASSETS, case_sensitive=False), 
-    help='If generating, specific assets that should be generated')
-@click.option('-d', '--diagrams', is_flag=True, help='OBSOLETE. Use --generate')
-@click.option('-w', '--webwork', is_flag=True, default=False, help='OBSOLETE. Use --generate')
-@click.option('-f', '--format', type=click.Choice(['html','latex','pdf']),
-              help='OBSOLETE. Update your project.ptx target instead.')
-@click.option('-i', '--input', 'source', type=click.Path(),
-              help='OBSOLETE. Update your project.ptx target instead.')
-@click.option('-o', '--output', type=click.Path(),
-              help='OBSOLETE. Update your project.ptx target instead.')
-@click.option('-p', '--publication', type=click.Path(), default=None,
-              help='OBSOLETE. Update your project.ptx target instead.')
-@click.option('-x', '--xsl', type=click.Path(), default=None,
-              help='OBSOLETE. Update your project.ptx target instead.')
-@click.option('--stringparam', nargs=2, multiple=True,
-              help='OBSOLETE. Update your project.ptx target instead.')
-def build(target, format, source, output, stringparam, xsl, publication, clean, generate,
-    webwork, diagrams,):
+    help='Generates assets for target.  -g [asset] will generate the specific assets given.')
+def build(target, clean, generate):
     """
     Build [TARGET] according to settings specified by project.ptx.
 
@@ -261,35 +262,17 @@ def build(target, format, source, output, stringparam, xsl, publication, clean,
     to non-Python executables may be set in project.ptx. For more details,
     consult the PreTeXt Guide: https://pretextbook.org/documentation.html
     """
-    if diagrams or webwork:
-        log.critical("Command used an asset option that is now obsolete.")
-        log.critical("Assets are now generated with `pretext generate` or `pretext build -g`.")
-        log.critical("Cancelling build. Check `--help` for details.")
-        return
-    if source or output or format or len(stringparam)>0 or xsl:
-        log.critical("Customizing target settings on the command line is no longer supported")
-        log.critical("Edit your `project.ptx` instead.")
-        log.critical("Cancelling build. Check `--help` for details.")
-        return
     target_name = target
-    # set up stringparams as dictionary:
-    if len(stringparam) > 0:
-        stringparams = {p[0] : p[1] for p in stringparam}
-    else:
-        stringparams = None
-    if utils.project_path() is None:
-        log.critical("No project.ptx manifest was found.")
-        log.critical("Use `pretext new` to create a new project or `pretext init` to update existing project for use with the CLI.")
+    if utils.no_project(task="build"):
         return
     project = Project()
-    if target_name is None:
-        log.info(f"Since no build target was supplied, the first target of the "+
-                    "project.ptx manifest will be built.")
     target = project.target(name=target_name)
+    if target_name is None:
+        log.info(f'Since no build target was supplied, the first target of the project.ptx manifest ({target.name()}) will be built.')
+    target_name = target.name()
     if target is None:
-        log.critical("Build target could not be found in project.ptx manifest.")
-        log.critical(f"Possible build targets are: {project.target_names()}")
-        log.critical("Exiting without completing task.")
+        utils.show_target_hints(target_name, project, task="build")
+        log.critical("Exiting without completing build.")
         return
     if generate=='ALL':
         log.info("Genearting all assets in default formats.")
@@ -302,41 +285,58 @@ def build(target, format, source, output, stringparam, xsl, publication, clean,
         log.warning("(previously generated assets will be used if they exist).")
         log.warning("To generate these assets before building, run `pretext build -g`.")
     project.build(target_name,clean)
+    log.info("")
 
 # pretext generate
-@main.command(short_help="Generate assets for specified target", 
+@main.command(short_help="Generate specified assets for specified target", 
     context_settings=CONTEXT_SETTINGS)
 @click.argument('assets', default="ALL",
     type=click.Choice(ASSETS, case_sensitive=False))
 @click.option(
-    '-t', '--target', type=click.STRING, help="Name of target to generate assets for (uses default target if not specified).")
+    '-t', '--target', type=click.STRING, help="Name of target to generate assets for (if not specified, first target from manifest is used).")
 @click.option(
     '-x', '--xmlid', type=click.STRING, help="xml:id of element to be generated.")
 @click.option('--all-formats', is_flag=True, default=False, 
-    help='Generate all possible asset formats rather than the defaults for the given target.')
+    help='Generate all possible asset formats rather than just the defaults for the specified target.')
 def generate(assets:str, target:Optional[str], all_formats:bool, xmlid:Optional[str]):
     """
-    Generate specified (or all) assets for the default target. Asset "generation" is typically
+    Generate specified (or all) assets for the default target (first target in "project.ptx"). Asset "generation" is typically
     slower and performed less frequently than "building" a project, but is
-    required for many PreTeXt features such as latex-image.
+    required for many PreTeXt features such as webwork and latex-image.
 
     Certain assets may require installations not included with the CLI, or internet
     access to external servers. Command-line paths
     to non-Python executables may be set in project.ptx. For more details,
     consult the PreTeXt Guide: https://pretextbook.org/documentation.html
     """
+    if utils.no_project(task="generate assets for"):
+        return
     project = Project()
     target_name = target
-    if assets == 'ALL':
-        log.info("Genearting all assets in default formats.")
-        project.generate(target_name,xmlid=xmlid)
+    target = project.target(name=target_name)
+    if target_name == None:
+        log.info(f'Since no target was specified with the -t flag, we will generate assets for the first target in the manifest ({target.name()}).')
+    if target is None:
+        utils.show_target_hints(target_name, project, task="generating assets for")
+        log.critical("Exiting without generating any assets.")
+        return
+    if all_formats and assets == 'ALL':
+        log.info(f'Generating all assets in all asset formats for the target "{target.name()}".')
+        project.generate(target.name(), all_formats=True, xmlid=xmlid)
+    elif all_formats:
+        log.info(f'Generating only {assets} assets in all asset formats for the target "{target.name()}".')
+        project.generate(target.name(), asset_list=[assets], all_formats=True, xmlid=xmlid)
+    elif assets == 'ALL':
+        log.info(f'Genearting all assets in default formats for the target "{target.name()}".')
+        project.generate(target.name(),xmlid=xmlid)
     else:
-        log.info(f"Generating only {assets} assets.")
-        project.generate(target_name,asset_list=[assets],xmlid=xmlid)
+        log.info(f'Generating only {assets} assets in default formats for the target "{target.name()}".')
+        project.generate(target.name(),asset_list=[assets],xmlid=xmlid)
+    log.info("")
 
 
 # pretext view
-@main.command(short_help="Preview specified target in your browser.", 
+@main.command(short_help="Preview specified target based on its format.",
     context_settings=CONTEXT_SETTINGS)
 @click.argument('target', required=False)
 @click.option(
@@ -346,7 +346,8 @@ def generate(assets:str, target:Optional[str], all_formats:bool, xmlid:Optional[
     default='private',
     show_default=True,
     help="""
-    Choose whether or not to allow other computers on your local network
+    If running a local server,
+    choose whether or not to allow other computers on your local network
     to access your documents using your IP address. (Ignored when used
     in CoCalc, which works automatically.)
     """)
@@ -355,14 +356,15 @@ def generate(assets:str, target:Optional[str], all_formats:bool, xmlid:Optional[
     '--port',
     type=click.INT,
     help="""
-    Choose which port to use for the local server.
+    If running a local server,
+    choose which port to use.
     """)
 @click.option(
     '-d',
     '--directory',
     type=click.Path(),
     help="""
-    Serve files from provided directory
+    Run local server for provided directory (does not require a PreTeXt project)
     """)
 @click.option('-w', '--watch', is_flag=True, help="""
     Run a build before starting server, and then
@@ -373,28 +375,36 @@ def generate(assets:str, target:Optional[str], all_formats:bool, xmlid:Optional[
     subsets of projects.
     """)
 @click.option('-b', '--build', is_flag=True, help="""
-    Run a build before starting server.
+    Run a build before viewing.
     """)
 @click.option(
     '-g', '--generate', is_flag=False, flag_value="ALL", default=None,
     type=click.Choice(ASSETS, case_sensitive=False), 
-    help='If generating, specific assets that should be generated')
-def view(target:str,access:str,port:Optional[int],directory:str,watch:bool,build:bool,generate:Optional[str]):
+    help='Generate all or specific assets before viewing')
+@click.option(
+    '--no-launch', is_flag=True,
+    help='By default, pretext view tries to launch the default application to view the specified target.  Setting this suppresses this behavior.'
+)
+def view(target:str,access:str,port:Optional[int],directory:str,watch:bool,build:bool,generate:Optional[str],no_launch:bool):
     """
     Starts a local server to preview built PreTeXt documents in your browser.
     TARGET is the name of the <target/> defined in `project.ptx`.
     """
+    # Easter egg to spin up a local server at a specified directory:
     if directory is not None:
         port = port or 8000
-        utils.run_server(Path(directory),access,port)
+        utils.run_server(Path(directory),access,port,no_launch=no_launch)
+        return
+    if utils.no_project(task="view the output for"):
         return
     target_name=target
     project = Project()
     target = project.target(name=target_name)
-    port = port or target.port()
     if target is None:
-        log.error(f"Target `{target_name}` could not be found.")
+        utils.show_target_hints(target_name, project, task="view")
+        log.critical("Exiting.")
         return
+    port = port or target.port()
     if generate == "ALL":
         log.info("Generating all assets in default formats.")
         project.generate(target_name)
@@ -404,7 +414,8 @@ def view(target:str,access:str,port:Optional[int],directory:str,watch:bool,build
     if build or watch:
         log.info("Building target.")
         project.build(target_name)
-    project.view(target_name,access,port,watch)
+    project.view(target_name,access,port,watch,no_launch)
+    log.info("")
 
 
 # pretext deploy
@@ -420,27 +431,15 @@ def deploy(target,update_source):
     properly configured with GitHub and GitHub Pages. Deployed
     files will live in `docs` subdirectory of project.
     """
+    if utils.no_project(task="deploy"):
+        return
     target_name = target
     project = Project()
+    target = project.target(name=target_name)
+    if target is None or target.format() != "html":
+        log.critical("Target could not be found in project.ptx manifest.")
+        log.critical(f"Possible html targets to deploy are: {project.target_names('html')}") #only list targets with html format.
+        log.critical("Exiting without completing task.")
+        return
     project.deploy(target_name,update_source)
-
-# pretext publish
-@main.command(short_help="OBSOLETE: use deploy",
-    context_settings=CONTEXT_SETTINGS)
-@click.argument('target', required=False)
-@click.option(
-    '-m',
-    '--commit-message',
-    default="Update to PreTeXt project source.",
-    show_default=True,
-    help="""
-    Customize message to leave on Git commit for source updates.
-    """)
-@click.pass_context
-def publish(ctx,target,commit_message):
-    """
-    OBSOLETE in favor of `deploy` command. Will be
-    removed in a future version.
-    """
-    log.error("`pretext publish` command is OBSOLETE.")
-    log.error("Use `pretext deploy` next time.")
+    log.info ("")
