@@ -3,7 +3,6 @@ import os
 import logging
 from . import utils, core
 from pathlib import Path
-import sys
 from typing import Optional
 
 # Get access to logger
@@ -16,6 +15,8 @@ def latex_image(ptxfile:Path, pub_file:Path, output:Path, params, target_format,
         'pdf': None,
         'latex':  None,
         'html': ['svg'],
+        'epub': ['svg'],
+        'kindle': ['png'],
     }
     # set overwrite formats to all when appropriate
     if all_formats:
@@ -46,9 +47,11 @@ def latex_image(ptxfile:Path, pub_file:Path, output:Path, params, target_format,
                         method=pdf_method
                     )
                 except Exception as e:
-                    log.critical(e)
-                    log.debug(f"Critical error info:\n****\n", exc_info=True)
-                    sys.exit('Failed to generate latex-image elements.  Exiting...')
+                    log.error(e)
+                    log.debug(f"Exception info:\n##################\n", exc_info=True)
+                    log.info('##################')
+                    log.error('Failed to generate some latex-image elements.  Check your source and partial output to diagnose the issue.')
+                    log.warning('Continuing...')
     else:
         log.info("Note: No latex-image elements found.")
 
@@ -59,6 +62,8 @@ def sageplot(ptxfile:Path, pub_file:Path, output:Path, params, target_format, xm
         'pdf': ['pdf','png'],
         'latex':  ['pdf','png'],
         'html': ['html','svg'],
+        'epub': ['svg'],
+        'kindle': ['png'],
     }
     # set overwrite formats to all when appropriate
     if all_formats:
@@ -87,9 +92,11 @@ def sageplot(ptxfile:Path, pub_file:Path, output:Path, params, target_format, xm
                         outformat=outformat
                     )
             except Exception as e:
-                log.critical(e)
-                log.debug(f"Critical error info:\n****\n", exc_info=True)
-                sys.exit('Failed to generate sageplot elements.  Exiting...')
+                log.error(e)
+                log.debug(f"Exception info:\n##################\n", exc_info=True)
+                log.info('##################')
+                log.error('Failed to generate some sageplot elements.  Check your source and partial output to diagnose the issue.')
+                log.warning('Continuing...')
     else:
         log.info("Note: No sageplot elements found.")
 
@@ -100,6 +107,8 @@ def asymptote(ptxfile:Path, pub_file:Path, output:Path, params, target_format, x
         'pdf': ['pdf'],
         'latex':  ['pdf'],
         'html': ['html'],
+        'epub': ['svg'],
+        'kindle': ['png'],
     }
     # set overwrite formats to all when appropriate
     if all_formats:
@@ -126,9 +135,11 @@ def asymptote(ptxfile:Path, pub_file:Path, output:Path, params, target_format, x
                         method='server'
                     )
             except Exception as e:
-                log.critical(e)
-                log.debug(f"Critical error info:\n****\n", exc_info=True)
-                sys.exit('Failed to generate asymptote elements.  Exiting...')
+                log.error(e)
+                log.debug(f"Exception info:\n##################\n", exc_info=True)
+                log.info('##################')
+                log.error('Failed to generate some asymptote elements. Check your source and partial output to diagnose the issue.')
+                log.warning('Continuing...')
     else:
         log.info("Note: No asymptote elements found.")
 
@@ -157,7 +168,8 @@ def interactive(ptxfile:Path, pub_file:Path, output:Path, params, xmlid_root):
             except Exception as e:
                 log.error("Failed to generate interactive element previews. Check debug log for info.")
                 log.debug(e)
-                log.debug(f"Critical error info:\n****\n", exc_info=True)
+                log.debug(f"Exception info:\n##################\n", exc_info=True)
+                log.info('##################')
                 log.warning('Continuing...')
     else:
         log.info("Note: No interactive elements found.")
@@ -183,9 +195,11 @@ def youtube(ptxfile:Path, pub_file:Path, output:Path, params, xmlid_root):
                     dest_dir=image_output.as_posix(),
                 )
             except Exception as e:
-                log.critical(e)
-                log.debug(f"Critical error info:\n****\n", exc_info=True)
-                sys.exit('Failed to genereate youtube video previews.  Exiting...')
+                log.error(e)
+                log.debug(f"Exception info:\n##################\n", exc_info=True)
+                log.info('##################')
+                log.error('Failed to genereate some youtube video previews. Check your source and partial output to diagnose the issue.')
+                log.warning('Continuing...')
     else:
         log.info("Note: No video@youtube elements found.")
 
@@ -211,9 +225,11 @@ def webwork(ptxfile:Path, pub_file:Path, output:Path, params):
                     server_params=None
                 )
             except Exception as e:
-                log.critical(e)
-                log.debug(f"Critical error info:\n****\n", exc_info=True)
-                sys.exit('Failed to generate webwork-representations file.  Exiting...')
+                log.error(e)
+                log.debug(f"Exception info:\n##################\n", exc_info=True)
+                log.info('##################')
+                log.error('Failed to generate the webwork-representations file.  Check your source and partial output to diagnose the issue.')
+                log.warning('Continuing...')
     else:
         log.info("Note: No webwork elements found.")
 
@@ -225,12 +241,6 @@ def codelens(ptxfile:Path, pub_file:Path, output:Path, params, xmlid_root):
     for _ in range(20):
         source_xml.xinclude()
     if len(source_xml.xpath("//program[@interactive = 'codelens']")) > 0:
-        try:
-            import runestone
-        except ImportError:
-            log.error("Runestone components are not installed. Skipping codelens trace.")
-            log.error("")
-            return
         trace_output = (output/'trace').resolve()
         os.makedirs(trace_output, exist_ok=True)
         log.info('Now generating codelens trace\n\n')
@@ -244,8 +254,10 @@ def codelens(ptxfile:Path, pub_file:Path, output:Path, params, xmlid_root):
                     dest_dir=trace_output.as_posix(),
                 )
             except Exception as e:
-                log.critical(e)
-                log.debug(f"Critical error info:\n****\n", exc_info=True)
-                sys.exit('Failed to generate codelens trace.  Exiting...')
+                log.error(e)
+                log.debug(f"Exception info:\n##################\n", exc_info=True)
+                log.info('##################')
+                log.error('Failed to generate codelens trace.')
+                log.warning('Continuing...')
     else:
         log.info("Note: No program elements using codelens found.")
