@@ -1,3 +1,4 @@
+from asyncio import subprocess
 import os
 import glob
 import random
@@ -9,13 +10,15 @@ from pathlib import Path
 import platform
 import socketserver
 import socket
+import subprocess
 import logging
 import threading
 import watchdog.events, watchdog.observers, time
+import webbrowser
 from typing import Optional
 from lxml import etree as ET
 
-from . import static, core
+from . import core
 
 # Get access to logger
 log = logging.getLogger('ptxlogger')
@@ -54,13 +57,14 @@ def project_path(dirpath:Optional[Path]=None) -> Path:
         return project_path(dirpath=dirpath.parent)
 
 def project_xml(dirpath:Optional[Path]=None) -> Path:
-    if dirpath==None:
+    if dirpath is None:
         dirpath = Path() # current directory
     if project_path(dirpath) is None:
-        project_manifest = static.path('templates','project.ptx')
+        with templates.resource_path('project.ptx') as project_manifest:
+            return ET.parse(project_manifest)
     else:
         project_manifest = project_path(dirpath) / 'project.ptx'
-    return ET.parse(project_manifest)
+        return ET.parse(project_manifest)
 
 def requirements_version(dirpath:Optional[Path]=None) -> str:
     if dirpath==None:
@@ -121,7 +125,7 @@ def xml_syntax_is_valid(xmlfile:Path) -> bool:
 
 def xml_source_validates_against_schema(xmlfile:Path) -> bool:
     #get path to RelaxNG schema file:
-    schemarngfile = static.path('schema','pretext.rng')
+    schemarngfile = core.resources.path('schema','pretext.rng')
 
     # Open schemafile for validation:
     relaxng = ET.RelaxNG(file=schemarngfile)
@@ -145,6 +149,12 @@ def xml_source_validates_against_schema(xmlfile:Path) -> bool:
         return False
     return True
 
+def cocalc_project_id() -> Optional[str]:
+    try:
+        return json.loads(open('/home/user/.smc/info.json').read())['project_id']
+    except:
+        return None
+
 # watchdog handler for watching changes to source
 class HTMLRebuildHandler(watchdog.events.FileSystemEventHandler):
     def __init__(self,callback):
@@ -164,19 +174,18 @@ class HTMLRebuildHandler(watchdog.events.FileSystemEventHandler):
 # boilerplate to prevent overzealous caching by preview server, and
 # avoid port issues
 def binding_for_access(access="private"):
-    if Path("/home/user/.smc/info.json").is_file() or access=="public":
+    if access=="public" or cocalc_project_id() is not None:
         return "0.0.0.0"
     else:
         return "localhost"
 def url_for_access(access="private",port=8000):
-    if Path("/home/user/.smc/info.json").is_file():
-        project_id = json.loads(open('/home/user/.smc/info.json').read())['project_id']
-        return f"https://cocalc.com/{project_id}/server/{port}/"
+    if cocalc_project_id() is not None:
+        return f"https://cocalc.com/{cocalc_project_id()}/server/{port}/"
     elif access=='public':
         return f"http://{socket.gethostbyname(socket.gethostname())}:{port}"
     else:
         return f"http://localhost:{port}"
-def serve_forever(directory:Path,access="private",port=8000):
+def serve_forever(directory:Path,access="private",port=8000,no_launch:bool=False):
     log.info(f"Now preparing local server to preview directory `{directory}`.")
     log.info("  (Reminder: use `pretext deploy` to deploy your built project to a public")
     log.info("  GitHub Pages site that can be shared with readers who cannot access your")
@@ -202,8 +211,11 @@ def serve_forever(directory:Path,access="private",port=8000):
             with TCPServer((binding, port), RequestHandler) as httpd:
                 looking_for_port = False
                 url = url_for_access(access,port)
-                log.info(f"Success! Open the below url in a web browser to preview the most recent build of your project.")
+                log.info(f"Success! The most recent build of your project can be viewed in a web browser at the following url:")
                 log.info("    "+url)
+                if not no_launch:
+                    log.info(f"This page should open in a new tab automatically.")
+                    webbrowser.open(url)
                 log.info("Use [Ctrl]+[C] to halt the server.\n")
                 httpd.serve_forever()
         except OSError:
@@ -211,9 +223,9 @@ def serve_forever(directory:Path,access="private",port=8000):
             port = random.randint(49152,65535)
             log.warning(f"Trying port {port} instead.\n")
 
-def run_server(directory:Path,access:str,port:int,watch_directory:Optional[Path]=None,watch_callback=lambda:None):
+def run_server(directory:Path,access:str,port:int,watch_directory:Optional[Path]=None,watch_callback=lambda:None, no_launch:bool=False):
     binding = binding_for_access(access)
-    threading.Thread(target=lambda: serve_forever(directory,access,port),daemon=True).start()
+    threading.Thread(target=lambda: serve_forever(directory,access,port,no_launch),daemon=True).start()
     if watch_directory is not None:
         log.info(f"\nWatching for changes in `{watch_directory}` ...\n")
         event_handler = HTMLRebuildHandler(watch_callback)
@@ -253,7 +265,7 @@ def copy_custom_xsl(xsl_path: Path, output_dir: Path):
     log.debug(f"Copying all files in {xsl_dir} to {output_dir}")
     shutil.copytree(xsl_dir, output_dir, dirs_exist_ok=True)
     if not (Path(output_dir)/'entities.ent').exists(): # TODO stop this as it's copied in subdirectory
-        shutil.copyfile(static.path('xsl/entities.ent'),Path(output_dir)/'entities.ent')
+        shutil.copyfile(core.resources.path('xsl/entities.ent'),Path(output_dir)/'entities.ent')
     # expand each xsl file
     with working_directory(output_dir):
         for filename in glob.iglob('**',recursive=True):
@@ -268,7 +280,7 @@ def copy_custom_xsl(xsl_path: Path, output_dir: Path):
                 # maybe an xsl file is malformed, but let's continue in case it's unused
                 except Exception as e:
                     log.warning(f"Failed to expand {filename} due to {e}, continuing anyway...")
-    shutil.copytree(static.core_xsl_dir_path(),output_dir/"core")
+    shutil.copytree(core.resources.path('xsl'),output_dir/"core")
 
 def check_executable(exec_name:str):
     try:
@@ -328,6 +340,46 @@ def check_asset_execs(element, outformats=None):
             #print installation hints based on operating system and missing program.
             log.info(install_hints[required_exec][platform.system()])
 
+def no_project(task:str) -> bool:
+    '''
+    Standard messages to be displayed when no project.ptx is found, customized by the "task" to be preformed.
+    '''
+    if project_path() is None:
+        log.critical(f"Before you can {task} your PreTeXt project, you must be in a (sub)directory initialized with a project.ptx manifest.")
+        log.critical("Move to such a directory, use `pretext new` to create a new project, or `pretext init` to update existing project for use with the CLI.")
+        return True
+    return False
+
+def show_target_hints(target_format:str, project, task:str):
+    '''
+    This will give the user hints about why they have provided a bad target and make helpful suggestions for them to fix the problem.  We will only run this function when the target_name is not the name in any target in project.ptx.
+    '''
+    # just in case this was called in the wrong place:
+    if project.target(name=target_format) is not None:
+        return
+    # Otherwise continue with hints:
+    log.critical(f'There is not a target named "{target_format}" for this project.ptx manifest.')
+    if target_format in ['html', 'pdf', 'latex','epub','kindle','braille']:
+        target_names = project.target_names(target_format)
+        if len(target_names) == 1:
+            log.info(f'However, the target named "{target_names[0]}" has "{target_format}" as its format.  Try to {task} that instead or edit your project.ptx manifest.')
+        elif len(target_names) > 1:
+            log.info(f'However, the targets with names {target_names} have "{target_format}" as their format.  Try to {task} one of those instead or edit your project.ptx manifest.')
+        if target_format in ['epub', 'kindle']:
+            log.info(f"Instructions for setting up a target with the {target_format} format, including the external programs required, can be found in the PreTeXt guide: https://pretextbook.org/doc/guide/html/epub.html")
+    else:
+        log.info(f"The available targets to {task} are named: {project.target_names()}.  Try to {task} on of those instead or edition your proejct.ptx manifest.")
+
+def npm_install():
+    with working_directory(core.resources.path("script", "mjsre")):
+        log.info("Attempting to install/update required node packages.")
+        try:
+            subprocess.run('npm install', shell=True)
+        except Exception as e:
+            log.critical("Unable to install required npm packages.  Please see the documentation.")
+            log.critical(e)
+            log.debug("", exc_info=True)
+
 def remove_path(path:Path):
     if path.is_file() or path.is_symlink():
         path.unlink()  # remove the file
